import pystan
import numpy as np
import matplotlib.pyplot as plt
import scipy.stats as ss
import seaborn as sb
import pickle
import tkinter as tk


sm = pystan.StanModel(file = 'SDT.stan')
with open('sm.pkl', 'wb') as f:
    pickle.dump(sm, f)
print('OK')

 
def CalcMAPEst(samples, a = 0.05, n_points = 2000):
    """
        Calculatte a MAP estimate from a KDE graph on [Lp, Up]
        Lp and Up are 100*a/2 and 100(1-a/2) percentile points of samples
    """
    Lp, Up = np.percentile(samples, [100 * a/2, 100 * (1 - a/2)])  # import numpy as np
    coord = np.linspace(Lp, Up, n_points)
    est_pdf = ss.gaussian_kde(samples).pdf(coord)  # import scipy.stats as ss
    map_idx = np.argmax(est_pdf)
    MAP_Est = coord[map_idx]      
    return MAP_Est, est_pdf[map_idx]

n_simu = 1000
mu_means = []
mu_meds = []
mu_maps = []
sgm_means = []
sgm_meds = []
sgm_maps = []

s_sgm = 1.101
s_mu = 1.726

c = [-1.489, -0.554, 0.198, 0.612, 1.983, 3.751]

K = len(c) + 1
n_st = 207  # 3 * 69

def judge(v):
    if v < c[0]:
        return 0
    elif v < c[1]:
        return 1
    elif v < c[2]:
        return 2
    elif v < c[3]:
        return 3
    elif v < c[4]:
        return 4
    elif v < c[5]:
        return 5
    else:
        return 6

root = tk.Tk()
root.title(f'simu_n100C7.py')
cnvs = tk.Canvas(width = 700, height = 300, background = '#00ffff')
cnvs.pack()
cnvs.create_text(350, 150, text = '', font = ('', 100), fill = 'blue',
                 tags = 'MyText')

for t in range(n_simu):
    #print(f't = {1+t}/{n_simu}     ', end = '\r' )
    
    cnvs.itemconfig('MyText', text = f'{t+1}/{n_simu}')
    cnvs.update()
    
    d_noise = np.zeros(K, dtype = int)
    d_signal = np.zeros(K, dtype = int)
    for t in range(n_st):
        v = ss.norm.rvs(0, 1)
        d_noise[judge(v)] += 1
        v = ss.norm.rvs(s_mu, s_sgm)
        d_signal[judge(v)]  += 1

    Data = {'K':K, 'ResNoise':d_noise, 'ResSignal':d_signal}
    with open('sm.pkl', 'rb') as f:
        sm = pickle.load(f)
    fit = sm.sampling(data = Data, iter = 10000, n_jobs = 1)

    mu_means.append(fit['mu_s'].mean())
    mu_meds.append(np.median(fit['mu_s']))
    mu_maps.append(CalcMAPEst(fit['mu_s'])[0])
    sgm_means.append(fit['sgm_s'].mean())
    sgm_meds.append(np.median(fit['sgm_s']))
    sgm_maps.append(CalcMAPEst(fit['sgm_s'])[0])

root.destroy()
print('\nSimulation ended.')

Y = sgm_means + sgm_meds + sgm_maps
Grp = ['mean']*len(sgm_means) + ['median']*len(sgm_meds) + ['map']*len(sgm_maps)
BPData = {'X':Grp, 'Y':Y}
Xmin, Xmax, Ymin, Ymax = sb.violinplot(x = 'X', y = 'Y', data = BPData, inner = 'quartile').axis()
plt.hlines(y = s_sgm, xmin = Xmin, xmax = Xmax, label = f's_sgm({s_sgm})')
plt.legend()
plt.title(f'n_noise=n_signal={n_st},  7 Categories\nDistributions of Estimates of s_sgm')
plt.tight_layout()
plt.savefig(f'FigSgms_n{n_st}C7.png')
plt.show()

Y = mu_means + mu_meds + mu_maps
Grp = ['mean']*len(mu_means) + ['median']*len(mu_meds) + ['map']*len(mu_maps)
BPData = {'X':Grp, 'Y':Y}
Xmin, Xmax, Ymin, Ymax = sb.violinplot(x = 'X', y = 'Y', data = BPData, inner = 'quartile').axis()
plt.hlines(y = s_mu, xmin = Xmin, xmax = Xmax, label = f's_mu({s_mu})')
plt.legend()
plt.title(f'n_noise=n_signal={n_st},  7 Categories\nDistributions of Estimates of s_mu')
plt.tight_layout()
plt.savefig(f'FigMus_n{n_st}C7.png')
plt.show()
